<?php

#
# This file is part of the Netdeep Secure Firewall.
#
# Netdeep Secure is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Netdeep Secure is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Netdeep Secure; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# (c) 2015 - www.netdeep.com.br
#

/**
 * View.class [ HELPER MVC ]
 * Reponsável por carregar o template, povoar e exibir a view, povoar e incluir arquivos PHP no sistem.
 * Arquitetura MVC!
 */
class View {

    private  $Data;
    private  $Keys;
    private  $Values;
    private  $Template;

    /**
     * <b>Carregar Template View:</b> informe o caminho e o nome do arquivo que deseja carregar como view.
     * Não precisa informar extenção. O arquivo deve ter o formato view<b>.tpl.html</b>
     * @param STRING $Template = Caminho / Nome_do_arquivo
     */
    public  function Load($Template) {
        $this->Template = REQUIRE_PATH . DIRECTORY_SEPARATOR .'_tpl' .DIRECTORY_SEPARATOR. (string) $Template;
        $this->Template = file_get_contents($this->Template . '.tpl.html');
    }

    /**
     * <b>Exibir Template View:</b> Execute um foreach com um getResult() do seu model e informe o envelope
     * neste método para configurar a view. Não esqueça de carregar a view acima do foreach com o método Load.
     * @param array $Data = Array com dados obtidos
     */
    public  function Show(array $Data) {
        $this->setKeys($Data);
        $this->setValues();
        $this->ShowView();
    }

    /**
     * <b>Carregar PHP View:</b> Tendo um arquivo PHP com echo em variáveis extraídas, utilize esse método
     * para incluir, povoar e exibir o mesmo. Basta informar o caminho do arquivo<b>.inc.php</b> e um
     * envelope de dados dentro de um foreach!
     * @param STRING $File = Caminho / Nome_do_arquivo
     * @param ARRAY $Data = Array com dados obtidos
     */
    public  function Request($File, array $Data) {
        extract($Data);
        require("{$File}.inc.php");		
    }

    /*
     * ***************************************
     * **********  PRIVATE METHODS  **********
     * ***************************************
     */

    //Executa o tratamento dos campos para substituição de chaves na view.
    private  function setKeys($Data) {
        $this->Data = $Data;
		$this->Data['HOME'] = HOME;
		
        $this->Keys = explode('&', '#' . implode("#&#", array_keys($this->Data)) . '#');
		$this->Keys[] = '#HOME#';
		
    }

    //Obtém os valores a serem inseridos nas chaves da view.
    private  function setValues() {
        $this->Values = array_values($this->Data);
    }

    //Exibe o template view com echo!
    private  function ShowView() {
        echo str_replace($this->Keys, $this->Values, $this->Template);
    }

}
